"use strict";

var _interopRequire = function (obj) { return obj && obj.__esModule ? obj["default"] : obj; };

var _createClass = (function () { function defineProperties(target, props) { for (var key in props) { var prop = props[key]; prop.configurable = true; if (prop.value) prop.writable = true; } Object.defineProperties(target, props); } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _classCallCheck = function (instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } };

Object.defineProperty(exports, "__esModule", {
    value: true
});

var React = _interopRequire(require("react"));

var _reactIntl = require("react-intl");

var ReactIntl = _interopRequire(_reactIntl);

var FormattedMessage = _reactIntl.FormattedMessage;
var IntlMixin = _reactIntl.IntlMixin;

var IntlDomainDatabase = exports.IntlDomainDatabase = (function () {
    function IntlDomainDatabase(loader) {
        _classCallCheck(this, IntlDomainDatabase);

        this.defaultDomains = {};
        this.locales = {};
        this.currentLocaleId = null;
        this.loader = loader;
        this.neededDomainIds = new Set();
    }

    _createClass(IntlDomainDatabase, {
        clearMessages: {
            value: function clearMessages() {
                this.locales = {};
            }
        },
        defaultMessages: {
            value: function defaultMessages(data) {
                this.defaultDomains[data.domainId] = data.messages;
            }
        },
        setLocale: {
            value: function setLocale(localeId) {
                this.currentLocaleId = localeId;
                return this.loadDomains(localeId);
            }
        },
        loadMessages: {
            value: function loadMessages(localeId, domainId) {
                var _this = this;

                var domains = this.locales[localeId];
                if (domains === undefined) {
                    this.locales[localeId] = domains = {};
                }
                var messages = domains[domainId];
                if (messages !== undefined) {
                    return Promise.resolve(messages);
                }
                if (this.loader === undefined) {
                    var _messages = this.defaultDomains[domainId];
                    if (!_messages) {
                        return Promise.reject(new Error("Loader not defined and cannot find domain: " + domainId));
                    }
                    return Promise.resolve(_messages);
                }
                return this.loader(localeId, domainId).then(function (messages) {
                    var domainMessages = _this.defaultDomains[domainId];
                    if (!messages) {
                        if (!domainMessages) {
                            return Promise.reject(new Error("Unknown locale " + localeId + " or domain " + domainId));
                        }
                        return domainMessages;
                    }
                    if (domainMessages) {
                        messages = Object.assign(domainMessages, messages);
                    }
                    domains[domainId] = messages;
                    return messages;
                });
            }
        },
        loadDomains: {
            value: function loadDomains(localeId) {
                var promises = [];
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = this.neededDomainIds[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var domainId = _step.value;

                        promises.push(this.loadMessages(localeId, domainId));
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator["return"]) {
                            _iterator["return"]();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }

                return Promise.all(promises);
            }
        },
        getMessages: {
            value: function getMessages(localeId, domainId) {
                var locale = this.locales[localeId];
                if (locale === undefined) {
                    return this.defaultDomains[domainId];
                }
                var domain = locale[domainId];
                if (domain === undefined) {
                    return this.defaultDomains[domainId];
                }
                return domain;
            }
        },
        getMessageById: {
            value: function getMessageById(localeId, domainId, path) {
                var messages = this.getMessages(localeId, domainId);
                var pathParts = path.split(".");
                var message = undefined;
                try {
                    message = pathParts.reduce(function (obj, pathPart) {
                        return obj[pathPart];
                    }, messages);
                } finally {
                    if (message === undefined) {
                        throw new ReferenceError("Could not find Intl message: " + path);
                    }
                }
                return message;
            }
        },
        makeFormatStr: {
            value: function makeFormatStr(domainId) {
                var db = this;
                this.neededDomainIds.add(domainId);
                return function (path, values) {
                    if (values === undefined) {
                        values = {};
                    }
                    // XXX formats support
                    var message = db.getMessageById(db.currentLocaleId, domainId, path);
                    var format = IntlMixin.getMessageFormat(message, db.currentLocaleId, {});
                    return format.format(values);
                };
            }
        },
        makeFormat: {
            value: function makeFormat(domainId) {
                var db = this;
                this.neededDomainIds.add(domainId);

                return React.createClass({
                    mixins: [ReactIntl.IntlMixin],
                    propTypes: {
                        tagName: React.PropTypes.string,
                        messageId: React.PropTypes.string
                    },
                    getMessageById: function getMessageById(path) {
                        return db.getMessageById(db.currentLocaleId, domainId, path);
                    },
                    render: function render() {
                        var props = Object.assign({}, this.props);
                        props.message = this.getMessageById(this.props.messageId);
                        props.locales = db.currentLocaleId;
                        delete props.messageId;
                        return React.createElement(FormattedMessage, props);
                    }
                });
            }
        }
    });

    return IntlDomainDatabase;
})();